"""CAS authentication backend.

Mostly copied from django_cas_ng.
"""

from __future__ import absolute_import
from __future__ import unicode_literals

from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.auth.backends import ModelBackend
from django.db import transaction
from django_cas_ng.signals import cas_user_authenticated
from django_cas_ng.utils import get_cas_client

from django_cas_binder.models import CASUser
from django_cas_binder.utils import get_free_username


USERNAME_TRIES_LIMIT = 1000


__all__ = ['CASBinderBackend']


class CASBinderBackend(ModelBackend):
    """CAS authentication backend"""

    def __init__(self):
        self.user_model = get_user_model()

    def clean_user_attributes_dict(self, attributes):
        def is_username_free(username):
            return not self.user_model.objects.filter(
                username=username).exists()

        attributes['username'] = get_free_username(
            attributes['username'], is_username_free, USERNAME_TRIES_LIMIT)

    @transaction.atomic
    def create_user_and_cas_user(self, universal_id, attributes):
        # user will have an "unusable" password
        u = self.user_model.objects.create_user(
            attributes['username'], attributes['email'])
        CASUser.objects.create(universal_id=universal_id, user=u)
        return u

    @transaction.atomic
    def update_user_attributes(self, user, attributes):
        update_attributes = getattr(
            settings, 'CAS_BINDER_UPDATE_USER_ATTRIBUTES', [])
        for attr in update_attributes:
            if attr in attributes:
                setattr(user, attr, attributes[attr])
        user.save()

    def authenticate(self, ticket, service, request):
        """Verifies CAS ticket and gets or creates user object"""
        client = get_cas_client(service_url=service)
        universal_id, attributes, pgtiou = client.verify_ticket(ticket)
        self.clean_user_attributes_dict(attributes)

        if attributes:
            request.session['attributes'] = attributes
        if not universal_id:
            return None

        try:
            user = CASUser.objects.get(universal_id=universal_id).user
            self.update_user_attributes(user, attributes)
            created = False
        except CASUser.DoesNotExist:
            # check if we want to create new users, if we don't fail auth
            if not settings.CAS_CREATE_USER:
                return None

            user = self.create_user_and_cas_user(universal_id, attributes)
            user.save()
            created = True

        if not self.user_can_authenticate(user):
            return None

        if pgtiou and settings.CAS_PROXY_CALLBACK:
            request.session['pgtiou'] = pgtiou

        # send the `cas_user_authenticated` signal
        cas_user_authenticated.send(
            sender=self,
            user=user,
            created=created,
            attributes=attributes,
            ticket=ticket,
            service=service,
        )
        return user

    def get_user(self, user_id):
        """Retrieve the user's entry in the user model if it exists"""

        try:
            return self.user_model.objects.get(pk=user_id)
        except self.user_model.DoesNotExist:
            return None

    def user_can_authenticate(self, user):
        """Added for compatibility with older Django versions (1.9), which
        don't implement user_can_authenticate.

        Field `active` is not considered here. CAS server should care for that.
        """
        return True



import django
from django.conf import settings


def setup_django():
    settings.configure(
        INSTALLED_APPS=[
            'django.contrib.auth',
            'django.contrib.contenttypes',
            'django_cas_binder'
        ],
        DATABASES={'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': ':memory:'
        }}

    )
    django.setup()



from django.db import transaction
from django_cas_ng.backends import User

from django_cas_binder.models import CASUser


@transaction.atomic
def assign_universal_ids(mapping):
    for email, universal_id in mapping.items():
        user = User.objects.filter(email=email).first()
        if user is not None:
            cas_user = CASUser.objects.filter(user=user).first()
            if cas_user is None:
                CASUser.objects.create(user=user, universal_id=universal_id)
            else:
                cas_user.universal_id = universal_id
                cas_user.save()



#!/usr/bin/env python
from distutils.core import setup


setup(
    name='django_cas_binder',
    version='1.6.1',
    description=(
        'A thin wrapper around django-cas-ng that allows to use identifier '
        'other than username.'
    ),
    author='Quantitative Engineering Design Inc.',
    author_email='',
    url='',
    packages=['django_cas_binder', 'django_cas_binder.migrations'],
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Framework :: Django',
    ],
    install_requires=[
        "django>=1.9.0"
    ]
)
